#include <sys/param.h>
#include <sys/ptrace.h>
#include <sys/user.h>
#include <machine/frame.h>
#include <machine/reg.h>

#if !defined (offsetof)
#define offsetof(TYPE, MEMBER) ((unsigned long) &((TYPE *)0)->MEMBER)
#endif

#define EAX	0
#define ESP	4
#define EIP	8

static int trapmap[] = {
	tEAX, tECX, tEDX, tEBX, tESP, tEBP, tESI, tEDI,
	tEIP, tEFLAGS, tCS, tSS, tDS, tES, tES, tES
};

static int syscmap[] = {
	sEAX, sECX, sEDX, sEBX, sESP, sEBP, sESI, sEDI,
	sEIP, sEFLAGS, sCS, sSS, sCS, sCS, sCS, sCS
};

static int *regmap;
static unsigned long *reg_offset;

static void
set_regoffset(int pid)
{
	static int fixed_kstack;
	caddr_t uaddr;

	reg_offset = (unsigned long *)
	    ptrace(PT_READ_U, pid, (caddr_t)
	     offsetof(struct user, u_kproc.kp_proc.p_md.md_regs), 0);

	if (!fixed_kstack) {
		uaddr = (caddr_t)ptrace(PT_READ_U, pid,
		    (caddr_t)offsetof(struct user, u_kproc.kp_proc.p_addr), 0);
		if ((caddr_t)reg_offset < uaddr)
			fixed_kstack = 1;
	}

	if (fixed_kstack)
		uaddr = (caddr_t)VM_MAXUSER_ADDRESS;

	reg_offset = (unsigned long *)((caddr_t)reg_offset - uaddr);
}

static int
fetch_register(int pid, int regno)
{
	int val;

	if (ptrace(PT_READ_U, pid, 
	    (caddr_t)offsetof(struct user, u_pcb.pcb_flags), 0) & FM_TRAP)
		regmap = trapmap;
	else
		regmap = syscmap;

	set_regoffset(pid);

	val = ptrace(PT_READ_U, pid,
	    (caddr_t)&reg_offset[regmap[regno]], 0);
	return val;
}

static void
store_register(int pid, int regno, int val)
{

	ptrace(PT_WRITE_U, pid, (caddr_t)&reg_offset[regmap[regno]], val);
}

int
md_fetch_eax(int pid)
{
	return(fetch_register(pid, EAX));
}

int
md_fetch_esp(int pid)
{
	return(fetch_register(pid, ESP));
}

int
md_fetch_eip(int pid)
{
	return(fetch_register(pid, EIP));
}

void
md_store_eax(int pid, int val)
{
	store_register(pid, EAX, val);
}

void
md_store_esp(int pid, int val)
{
	store_register(pid, ESP, val);
}

void
md_store_eip(int pid, int val)
{
	store_register(pid, EIP, val);
}
