#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ptrace.h>
#include <sys/syscall.h>
#include <unistd.h>

#include "ltrace.h"
#include "options.h"

/* Returns 1 if the sysnum may make a new child to be created
 * (ie, with fork() or vfork())
 * Returns 0 otherwise.
 */
int fork_p(int sysnum)
{
	return (sysnum == SYS_fork || sysnum == SYS_vfork);
}

/* Returns 1 if the sysnum may make the process exec other program
 */
int exec_p(int sysnum)
{
	return (sysnum == SYS_execve);
}

void trace_me(void)
{
	if (ptrace(PT_TRACE_ME, 0, 0, 0)<0) {
		perror("PT_TRACE_ME");
		exit(1);
	}
}

int trace_pid(pid_t pid)
{
	if (ptrace(PT_ATTACH, pid, 0, 0) < 0) {
		return -1;
	}
	return 0;
}

void untrace_pid(pid_t pid)
{
	ptrace(PT_DETACH, pid, 0, 0);
}

void continue_after_signal(pid_t pid, int signum)
{
	ptrace(PT_CONTINUE, pid, PT_ADDR_ANY, signum);
}

void continue_process(pid_t pid)
{
	continue_after_signal(pid, 0);
}

void continue_enabling_breakpoint(pid_t pid, struct breakpoint * sbp)
{
	insert_breakpoint(pid, sbp);
	continue_process(pid);
}
