#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/ptrace.h>
#include <sys/param.h>
#include <sys/user.h>
#include <stdlib.h>

#include "ltrace.h"

void continue_after_breakpoint(struct process *proc, struct breakpoint * sbp, int delete_it)
{
	struct user *u = NULL;
	delete_breakpoint(proc->pid, sbp);
	md_store_eip(proc->pid, (int)sbp->addr);
	if (delete_it) {
		continue_process(proc->pid);
	} else {
		proc->breakpoint_being_enabled = sbp;
		ptrace(PT_STEP, proc->pid, (caddr_t)1, 0);
	}
}

long gimme_arg(enum tof type, struct process * proc, int arg_num)
{
	if (arg_num==-1) {		/* return value */
		return md_fetch_eax(proc->pid);
	}

	if (type==LT_TOF_FUNCTION) {
		return ptrace(PT_READ_I, proc->pid, proc->stack_pointer+4*(arg_num+1), 0);
	} else if (type==LT_TOF_SYSCALL) {
		return ptrace(PT_READ_D, proc->pid, 4*arg_num, 0);
	} else {
		fprintf(stderr, "gimme_arg called with wrong arguments\n");
		exit(1);
	}

	return 0;
}

int umovestr(struct process * proc, void * addr, int len, void * laddr)
{
	long a;
	int i;
	int offset=0;

	while(offset<len) {
		a = ptrace(PT_READ_I, proc->pid, addr+offset, 0);
		for(i=0; i<sizeof(long); i++) {
			if (((char*)&a)[i] && offset+i < len) {
				*(char *)(laddr+offset+i) = ((char*)&a)[i];
			} else {
				*(char *)(laddr+offset+i) = '\0';
				return 0;
			}
		}
		offset += sizeof(long);
	}
	*(char *)(laddr+offset) = '\0';
	return 0;
}
